@@ -49,6 +49,7 @@
 #include "speedlimits_dialog.h"
 #include "toolbar.h"
 #include "menu_bar.h"
+#include "string_coalescer.h"
 
 #ifdef __WXMSW__
 #include <wx/module.h>
@@ -91,6 +92,7 @@ BEGIN_EVENT_TABLE(CMainFrame, wxFrame)
 	EVT_TOOL(XRCID("ID_TOOLBAR_LOCALTREEVIEW"), CMainFrame::OnToggleLocalTreeView)
 	EVT_TOOL(XRCID("ID_TOOLBAR_REMOTETREEVIEW"), CMainFrame::OnToggleRemoteTreeView)
 	EVT_TOOL(XRCID("ID_TOOLBAR_QUEUEVIEW"), CMainFrame::OnToggleQueueView)
+	EVT_MENU(XRCID("ID_VIEW_TOOLBAR"), CMainFrame::OnToggleToolBar)
 	EVT_MENU(XRCID("ID_VIEW_MESSAGELOG"), CMainFrame::OnToggleLogView)
 	EVT_MENU(XRCID("ID_VIEW_LOCALTREE"), CMainFrame::OnToggleLocalTreeView)
 	EVT_MENU(XRCID("ID_VIEW_REMOTETREE"), CMainFrame::OnToggleRemoteTreeView)
@@ -623,6 +625,14 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 	{
 		CWrapEngine::ClearCache();
 	}
+	else if (event.GetId() == XRCID("ID_COALESCER_CLEAR"))
+	{
+		ClearStringCoalescer();
+	}
+	else if (event.GetId() == XRCID("ID_COALESCER_BENCHMARK"))
+	{
+		BenchmarkStringCoalescer();
+	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_FILEEXISTS"))
 	{
 		CDefaultFileExistsDlg dlg;
@@ -640,8 +650,10 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		pDlg->Show();
 		pDlg->Delete();
 
-		m_pMenuBar->UpdateMenubarState();
-		m_pToolBar->UpdateToolbarState();
+		if (m_pMenuBar)
+			m_pMenuBar->UpdateMenubarState();
+		if (m_pToolBar)
+			m_pToolBar->UpdateToolbarState();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_VIEWHIDDEN"))
 	{
@@ -1025,8 +1037,14 @@ bool CMainFrame::CreateToolBar()
 #endif
 		SetToolBar(0);
 		delete m_pToolBar;
+		m_pToolBar = 0;
 	}
 
+#ifndef __WXMAC__
+	if (COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) != 0)
+		return true;
+#endif
+
 	m_pToolBar = CToolBar::Load(this);
 	if (!m_pToolBar)
 	{
@@ -1599,7 +1617,7 @@ void CMainFrame::ShowLocalTree()
 			continue;
 
 		controls->pLocalTreeViewPanel->SetHeader(controls->pLocalListViewPanel->DetachHeader());
-		wxSize size = controls->pLocalSplitter->GetClientSize();
+		
 		if (layout == 3 && swap)
 			controls->pLocalSplitter->SplitVertically(controls->pLocalListViewPanel, controls->pLocalTreeViewPanel);
 		else if (layout)
@@ -1655,7 +1673,7 @@ void CMainFrame::ShowRemoteTree()
 			continue;
 
 		controls->pRemoteTreeViewPanel->SetHeader(controls->pRemoteListViewPanel->DetachHeader());
-		wxSize size = controls->pRemoteSplitter->GetClientSize();
+
 		if (layout == 3 && !swap)
 			controls->pRemoteSplitter->SplitVertically(controls->pRemoteListViewPanel, controls->pRemoteTreeViewPanel);
 		else if (layout)
@@ -1701,13 +1719,9 @@ void CMainFrame::OnToggleQueueView(wxCommandEvent& event)
 	else
 	{
 		if (m_pBottomSplitter->IsSplit())
-		{
-			wxRect rect = m_pBottomSplitter->GetClientSize();
 			m_pBottomSplitter->Unsplit(m_pQueueLogSplitter);
-		}
 		else
 		{
-			wxRect rect = m_pBottomSplitter->GetClientSize();
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
 			m_pBottomSplitter->SplitHorizontally(m_pContextControl, m_pQueueLogSplitter);
 		}
@@ -1948,6 +1962,9 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 
 void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 {
+	if (!m_pToolBar)
+		return;
+
 	wxMenu *pMenu = CSiteManager::GetSitesMenu();
 	if (!pMenu)
 		return;
@@ -2328,6 +2345,9 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent& event)
 
 void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 {
+	if (!m_pToolBar)
+		return;
+
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TOOLBAR_COMPARISON_DROPDOWN"));
 	if (!pMenu)
 		return;
@@ -2352,7 +2372,7 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 
 void CMainFrame::ShowDropdownMenu(wxMenu* pMenu, wxToolBar* pToolBar, wxCommandEvent& event)
 {
-	#ifdef EVT_TOOL_DROPDOWN
+#ifdef EVT_TOOL_DROPDOWN
 	if (event.GetEventType() == wxEVT_COMMAND_TOOL_DROPDOWN_CLICKED)
 	{
 		pToolBar->SetDropdownMenu(event.GetId(), pMenu);
@@ -2712,3 +2732,15 @@ bool CMainFrame::ConnectToServer(const CServer &server, const CServerPath &path
 
 	return pState->Connect(server, path);
 }
+
+void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
+{
+	COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
+#ifdef __WXMAC__
+	if (m_pToolBar)
+		m_pToolBar->Show( event.IsChecked() );
+#else
+	CreateToolBar();
+	HandleResize();
+#endif
+}
